######!/usr/bin/env python
# -*- coding: utf-8 -*-

# Do divisions with Reals, not with integers
# Must be at the beginning of the file
from __future__ import division
from __future__ import absolute_import
from __future__ import print_function

# To do :
#	- test other projections, I have not tested it
#   - mask the frequency outside of min/max
#   - remove holes from the dem... TO BE TESTED
#   - remove holes from the min file
# End To Do

###### CHANGES
#    -2015/08/07 : Correct the stepping if two types of profiles (with or without multipoints) are asked in the same run
#                  Add options to choose the size of the graphic
#                  Add options to force the density bar color scale
#    -2015/08/06 : Save a shapefile with the points of the profil
#    -2015/08/04 : Multipoint profils
#    -2015/07/30 : correct bug in function findline to take in account if B>A
#	 -2015/07/30 : remove NaN from graphs
#    -2015/09/15 : Add swath on synthetic DEM
#
######  End CHANGES


############################################################################

## Import Python modules
## I have problems to install rasterio : it does not find gdal libraries... from kingchaos
modulesNames = ['sys', 'math', 'os', 'utm', 'shapely']
for module in modulesNames:
	try:
		# because we want to import using a variable, do it this way
		module_obj = __import__(module)
		# create a global object containging our module
		globals()[module] = module_obj
	except ImportError:
		#sys.exit(u"ERROR : Module " + module + " not present. \n\n Please, install it \
		raise ImportError(u"ERROR : Module " + module + " not present. \n\n Please, install it \
			      \n\n Edit the source code for more information")
from os import path, access, R_OK, mkdir         # W_OK for write permission.
from rasterstats import raster_stats, zonal_stats             # For stats on rasters
from shapely.geometry import Polygon, LineString, shape, box, MultiPolygon
from shapely import affinity
from shapely import speedups
if speedups.available:
	speedups.enable()
try:
	import numpy as np                               # need version 1.7 or higher
except ImportError:
	#sys.exit(u"ERROR : Module Numpy not present. \n\n Please, install it \
	raise ImportError(u"ERROR : Module Numpy not present. \n\n Please, install it \
		      \n\n Edit the source code for more information")
try:
	from osgeo import gdal, gdalnumeric, ogr, osr    # For GIS operations
	from osgeo.gdalconst import *
except ImportError:
	#sys.exit(u"ERROR : Module osgeo/gdal not present. \n\n Please, install it \
	raise ImportError(u"ERROR : Module osgeo/gdal not present. \n\n Please, install it \
		      \n\n Edit the source code for more information")
try:
	from progress.bar import Bar                     # For a progress bar in the terminal output
except ImportError:
	#sys.exit(u"ERROR : Module progress not present. \n\n Please, install it \
	raise ImportError(u"ERROR : Module progress not present. \n\n Please, install it \
		      \n\n Edit the source code for more information")

from .raster_tools import *
from .plotgraph import *
############################################################################

def makeshape(A, B, boxwidth, shp, srs, xstep = 'NULL', nx = 'NULL' , ny = 'NULL', beta = 'NULL'):
	"""
	Create a shapefile around a profile defined the points A and B, 
	with the boxwidth given
	
	INPUTS:
	   A(Xa,Ya) and B(Xb,Yb) = Points defining the line around which we build the shapefile
	   boxwidth = width of the box around the profile A-B
	   shp = path+name of the output shapefile
	   srs = coordinate system of the shapefile to create
	   xstep = size of the x-stepping in the unit of the DEM/shapefiles
	   shp = name or the shapefile that will be produced
	   nx = number of points in the direction normal to the main profile
	   ny = number of points in the direction of the main profile
	   beta = angle between the parallel and the transect A-B  
	
	OUTPUTS:
	   nx = number of points in the direction normal to the main profile
	   ny = number of points in the direction of the main profile
	   beta = angle between the parallel and the transect A-B
	   shapefile shp
	
	USAGE:
	  nsteps, ny, beta = makeshape(A, B, boxwidth, shpbox2, srs, 
	                               xstep = xsteps, [nx], [ny], [beta])
	  or:
	  makeshape(A, B, boxwidth, shpbox2, srs, 
	            xstep = 'NULL', [nx], [ny], [beta])
	
	"""
	
	# Calcul the length of the box
	xdistm = calc_length(A,B)
	if xstep != 'NULL':
		xstep = xstep
		nx = np.int(xdistm / xstep)
		ny = np.int(boxwidth / xstep)
		# beta is the angle between the parallel and the transect
		if beta == 'NULL':
			beta = math.atan((B[1] - A[1]) / (B[0] - A[0]))
	
	# create the corner points of the shapefile
	aa = np.empty(2)
	bb = np.empty(2)
	cc = np.empty(2)
	dd = np.empty(2)
	
	# Recalcul the coordinates of the corner :
	xxx = boxwidth / 2 * np.sin(beta)
	yyy = boxwidth / 2 * np.cos(beta)
	
	aa[0] = A[0] - xxx
	aa[1] = A[1] + yyy
	dd[0] = A[0] + xxx
	dd[1] = A[1] - yyy
	cc[0] = B[0] + xxx
	cc[1] = B[1] - yyy
	bb[0] = B[0] - xxx
	bb[1] = B[1] + yyy
	
	# from http://gis.stackexchange.com/questions/52705/how-to-write-shapely-geometries-to-shapefiles
	# Create the polygon to be used for the shapefile
	poly = Polygon([(aa[0],aa[1]), (bb[0],bb[1]), (cc[0],cc[1]), (dd[0],dd[1]), (aa[0],aa[1])])
	
	# Now convert it to a shapefile with OGR    
	driver = ogr.GetDriverByName('Esri Shapefile')
	ds = driver.CreateDataSource(shp)
	# Set the Reference
	layer = ds.CreateLayer('zoup', srs, ogr.wkbPolygon)
	# Add one attribute
	layer.CreateField(ogr.FieldDefn('id', ogr.OFTInteger))
	defn = layer.GetLayerDefn()
	
	## If there are multiple geometries, put the "for" loop here
	# Create a new feature (attribute and geometry)
	feat = ogr.Feature(defn)
	feat.SetField('id', 123)
	# Make a geometry, from Shapely object
	geom = ogr.CreateGeometryFromWkb(poly.wkb)
	feat.SetGeometry(geom)
	layer.CreateFeature(feat)
	feat = geom = None  # destroy these
	# Save and close everything
	ds = layer = feat = geom = None
	
	if xstep != 'NULL':
		#return nx, ny, x, y
		return nx, ny, beta
	else:
		return

############################################################################

def save_shape(A, B, shp, srs, iii, kkk = None):
	"""
	Create a shapefile around a profile defined the points A and B, 
	with the boxwidth given
	
	INPUTS:
	   A(Xa,Ya) and B(Xb,Yb) = Points defining the line around which we build the shapefile
	   shp = path+name of the output shapefile
	   srs = coordinate system of the shapefile to create
	   iii = profile number
	   kkk = sub-profile number
	
	OUTPUTS:
	   shapefile shp with the points used in the profile
	
	USAGE:
	   save_shape(A, B, shp, srs)
	
	"""

	## Create the shapefile points
	shp2 = shp[0:-4] + '_points.shp'
	#shp2 = 'points_' + shp[6:]
	print(u'   Saving points defining the profil in the shapefile %s' % shp2)
	# Now convert it to a shapefile with OGR    
	driver = ogr.GetDriverByName('Esri Shapefile')
	ds = driver.CreateDataSource(shp2)
	# Set the Reference
	layer = ds.CreateLayer('zoup', srs, ogr.wkbPoint)
	# Add one attribute
	layer.CreateField(ogr.FieldDefn('id', ogr.OFTInteger))
	layer.CreateField(ogr.FieldDefn('Name', ogr.OFTString))
	layer.CreateField(ogr.FieldDefn('X', ogr.OFTReal))
	layer.CreateField(ogr.FieldDefn('Y', ogr.OFTReal))
	
	defn = layer.GetLayerDefn()
	
	# create point A
	point = ogr.Geometry(ogr.wkbPoint)
	point.SetPoint(0, A[0], A[1])
	# Create feature
	feature = ogr.Feature(defn)
	feature.SetGeometry(point)
	feature.SetFID(1)
	feature.SetField('id', 1)
	if kkk == None:
		feature.SetField('Name', 'A_' + str(iii))
	else:
		feature.SetField('Name', 'A_' + str(iii) + '_' + str(kkk))
	# add the coordinates in the attribut table
	feature.SetField('X', A[0])
	feature.SetField('Y', A[1])	
	# Save feature
	layer.CreateFeature(feature)
	# Cleanup
	point.Destroy()
	feature.Destroy()

	# create point B
	point = ogr.Geometry(ogr.wkbPoint)
	point.SetPoint(0, B[0], B[1])
	# Create feature
	feature = ogr.Feature(defn)
	feature.SetGeometry(point)
	feature.SetFID(2)
	feature.SetField('id', 2)
	if kkk == None:
		feature.SetField('Name', 'B_' + str(iii))
	else:
		feature.SetField('Name', 'B_' + str(iii) + '_' + str(kkk))
	# add the coordinates in the attribut table
	feature.SetField('X', B[0])
	feature.SetField('Y', B[1])
	# Save feature
	layer.CreateFeature(feature)
	# Cleanup
	point.Destroy()
	feature.Destroy()

	# Save and close everything
	ds = layer = None
	
	return

############################################################################
		
def findline(A, B, Xi, Xii, inc, i, nsteps, dist):
	"""
	Function to find the sub-line to build the sub-shapefile
	
	INPUTS:
	   A = A coordinates (xai, yai)
	   B = B coordinates (xbi, ybi)
	   Xi = coordinates of the first point of the line
	   Xii = coordinates of the second point of the line
	   inc = distance between two points
	   i = step's number
	   nsteps = total number of steps
	   dist = nsteps-Array of distance along the profile.
	
	OUTPUTS:
	   Xi = coordinates of the first point of the line updated for the next step
	   Xii = coordinates of the second point of the line updated for the next step
	   dist = nsteps-Array of distance along the profile updated
	
	USAGE:
	   Xi, Xii, dist = findline(A, B, Xi, Xii, inc, i, nsteps, dist)
	
	Written by X. Robert, june 2014
	"""
	# beta is the angle between the parallel and the transect
	#beta = np.pi / 2 - math.radians(bearing(A[0], A[1], B[0], B[1]))
	# The previous line is true if coordinates in lat/long (deg)
	# For utm in meters, the true value is the next one
	beta = np.arctan((B[1]-A[1]) / (B[0]-A[0]))
	if B[0] > A[0]:
		beta = -beta
	if B[1] > A[1]:
		beta = -beta
	incx = inc
	incy = inc
	if A[0] > B[0]:
		incx = -incx
	if	A[1] > B[1]:
		incy = -incy
	if i == 0 :
		Xi = [A[0],A[1]]
		# 15/06/2015; XR; inc/2 replaced by inc	
		Xii = [Xi[0] + incx * np.cos(beta), Xi[1] + incy * np.sin(beta)]
	else:
		Xi = Xii
		Xii = [Xi[0] + incx * np.cos(beta), Xi[1] + incy * np.sin(beta)]
	if i > 0: dist[i] = dist[i-1] + calc_length((Xi[0], Xi[1]), (Xii[0], Xii[1]))
	
	return Xi, Xii, dist	

############################################################################

def main_xa(A, B, xsteps, boxwidth, binsize, title, shpbox, rasterfnme, srs, factor, 
            sizeplotx = None, sizeploty = None, densitymin = None, densitymax = None,
            ylimmin = None, ylimmax = None,
            corrnan = False, synthetic = False, iii = None, 
            multipoints = None, nbpointsint = None, kkk = None):
	"""
	Main function that extract the data and plot the graph for each sets of A and B
	
	INPUTS:
	   A = A coordinates (xai,yai)
	   B = B coordinates (xbi,ybi)
	   xsteps = steps for the distance stepping
	   boxwidth = width of the box inside which we extract the data
	   binsize = altitudes binsize to build the histogram/frequency of the altitudes
	   title = Title of the graphs/transects
	   shpbox = Name of the shapefile that define the box around the profile to extract
	   rasterfnme = Name of the raster from which we extract the profile
	   srs = Coordinate system of the input dem
	   factor = factor to apply to path from the projected dem measure system to km 
	            (generally = 1000)
	   corrnan = Flag to correct (True) or no (False) the graph from the Nan values
	   synthetic = True if synthetic dem from a SPEM
	   iii = transect's number, i.e. rank in the main list A or B
	   multipoints = flag to set a profile along a multiline with different points
	   nbpointsmulti = if multipoints, number of intermediary poitns
	   kkk = rank of the intermediary points
	
	OUTPUTS:
	   no variables
	
	USAGE:
	   main_xa(A, B, xsteps, boxwidth, binsize, title, shpbox, rasterfnme, srs, factor, [iii])
	
	Written by X. Robert - June 2014
	"""
	# build the line A-B
	xdist = calc_length(A,B) / factor
	
	print(u'   Profile between A(' + str(A[0]) + u', ' + str(A[1]) + u') and B(' + str(B[0]) + u', ' + str(B[1]) + u')')
	#print('   Length of the profile : ' + str(xdist) + ' km')
	print(u'   Build the box shapefile...  ')
		
	# Build the line shapefile
	lineshp = 'SHP_' + title + '/line_' + title + '_' + str(iii+1) + '.shp'
	if multipoints:
		lineshp = lineshp[0:-4] + '_sub_' + str(kkk+1) + '.shp'
	# from http://gis.stackexchange.com/questions/52705/how-to-write-shapely-geometries-to-shapefiles
	# Create the polygon to be used for the shapefile
	polyl = LineString([(A[0],A[1]), (B[0],B[1])])
	# Now convert it to a shapefile with OGR    
	driver = ogr.GetDriverByName('Esri Shapefile')
	ds = driver.CreateDataSource(lineshp)
	
	# Set the Reference
	# This is in srs_new
	layer = ds.CreateLayer('zoup', srs, ogr.wkbLineString)
	# Add one attribute
	layer.CreateField(ogr.FieldDefn('id', ogr.OFTInteger))
	defn = layer.GetLayerDefn()
	## If there are multiple geometries, put the "for" loop here
	# Create a new feature (attribute and geometry)
	feat = ogr.Feature(defn)
	feat.SetField('id', 123)
	# Make a geometry, from Shapely object
	geom = ogr.CreateGeometryFromWkb(polyl.wkb)
	feat.SetGeometry(geom)
	layer.CreateFeature(feat)
	feat = geom = None  # destroy these
	# Save and close everything
	ds = layer = feat = geom = None
	# Build the box shapefile around the main line
	shpbox2 = shpbox[0:-4] + '_'  + str(iii + 1) + '.shp'	
	# build the general shapefile from the [AB] line
	# and calcul the number of steps between A and B :
	nsteps, ny, beta = makeshape(A, B, boxwidth, shpbox2, srs, xstep = xsteps)
	# Save the points in a shapefile
	save_shape(A, B, shpbox2, srs, iii, kkk)
	
	# extract the statistics data for the whole box:
	stats = zonal_stats(shpbox2,
    	            	rasterfnme,
        	            stats=['min', 'max'])                         

	# initiate the tables for the heat map
	dist = np.empty(nsteps)
	inc = xsteps
	dist = np.zeros(nsteps)
	dist[0] = 0
	altimin = [f['min'] for f in stats][0]
	altimax = [f['max'] for f in stats][0]
	Xi = [0,0]
	Xii = [0,0]
	# set minimum altitude to 0 if it is inferior
	if altimin < 0:
		altimin = 0
	if altimax == None or altimax < altimin:
		raise NameError(u"   ERROR : There is a problem with the values in the DEM. \n"
		         u"   Check it! ")
	altirg = np.arange(altimin, altimax + binsize, binsize)	
	falti = np.zeros((dist.shape[0], altirg.shape[0]-1))
	statslines = np.empty((dist.shape[0],5))
	
	print(u'   Data extraction...')
	if multipoints:
		sub = 'sub-'
	else:
		sub = ''
	# Define the progress-bar that is output in the terminal window
	#bar = Bar('   Processing %sprofile %i' % sub, (iii+1), max = nsteps, suffix = '%(index)d / %(max)d ')
	if multipoints:
		bar = Bar(u'   Processing profile %i.%i' % ((iii+1), (kkk+1)), max = nsteps+1, suffix = '%(index)d / %(max)d ')
	else:
		bar = Bar(u'   Processing profile %i' % (iii+1), max = nsteps+1, suffix = '%(index)d / %(max)d ')
	# Stepping along the line
	for i in range (0,nsteps):
		
		# first add a step in the progress-bar
		bar.next()

		# build the shapefile between A+i*istep and A+(i+1)*istep
		# find the line:
		Xi, Xii, dist = findline(A, B, Xi, Xii, inc, i, nsteps, dist)
		
		# Store the sub-shapefile in TMP/
		# If they are needed, comment the end of the code in the function clean() 
		#                     that erase them when the run is finished
		shp = 'TMP/shp' + str(i) + '.shp'
		# build the sub-shapefile between Xi and Xii
		junk = makeshape(Xi, Xii, boxwidth, shp, srs, beta = beta)
		
		# calcul the distance from A
		statslines[i,0] = calc_length(A, Xi)
		
		# calcul the stats
		# Zonal_stats replace raster_stats.
		stats = zonal_stats(vectors = shp,
			                raster = rasterfnme, 
			                nodata_value = -9999,
			                raster_out = True,
		                    stats = ['min', 'max', 'median', 'mean', 'count'])
		# report it in a distance table statslines as followed:
		#	- row 0 = distance
		#	- row 1 = min altitude value of the shapefile
		#	- row 2 = max altitude value of the shapefile
		#	- row 3 = median altitude value of the shapefile
		#	- row 4 = mean altitude value of the shapefile
		statslines[i,1] = [f['min'] for f in stats][0]
		statslines[i,2] = [f['max'] for f in stats][0]
		statslines[i,3] = [f['median'] for f in stats][0]
		statslines[i,4] = [f['mean'] for f in stats][0]
		# Compute the altitude histogram
		try:
			#falti[i,:], alt = np.histogram([f['mini_raster_array'] for f in stats][0], altirg, density = True)
			falti[i,:], alt = np.histogram([f['mini_raster_array'] for f in stats][0].compressed(), altirg, density = True)
		except KeyError as e:
			print(repr(e))
			raise NameError(u'Well... This error could come from a mismatch between one point of the profil and the region of the DEM \n'
			          u'Please, check if the ongoing profile is well defined (Plot few of the TMP/shapefile on the DEM to check that)')
			
		# norm the altitude density
		falti[i,:] = falti[i,:] * np.diff(alt)
		
	bar.next()
	# Finish the progress-bar
	bar.finish()
	
	# Change the altitude def for the plot : 
	# choose the middle of the interval between the step i and the step j+1
	alti = [(altirg[hh] + altirg[hh+1])/2 for hh in range (0, altirg.shape[0]-1)]
	# initiate data:
	data = np.empty([falti.size,3])
	# change the falti table to a table with 3 columns : dist, altitude, frequency
	# to be able to plot it as a heatmap (see the plt.hexbin plot).
	data[:,0] = np.repeat(dist, falti.shape[1])
	data[:,1] = np.tile(alti, falti.shape[0])
	data[:,2] = falti.flatten()
	# OLD:
	#for i in range (0,falti.shape[0]):		 
		#dist = np.vstack((dist,dist))
		#alti = np.vstack((alti,alti))
		#for j in range (0,falti.shape[1]):		
			#data[j+i*falti.shape[1],0] = dist[i]
			#data[j+i*falti.shape[1],1] = (altirg[j] + altirg[j+1])/2
			#data[j+i*falti.shape[1],2] = falti[i,j]
					
	if multipoints:
		return statslines, data, xdist
	else:
		# mask the data where it is out of the bounds statslines[:,1] and statslines[:,2]
		# mask the data where it is null (set null value to nan)
		datamask1 = np.ma.masked_where(data[:,2] == 0, data[:,2])
		# mask the data where it is closed to 1 to get a colorbar not overprinted by only 1 value
		datamask = np.ma.masked_where(datamask1 == 1, datamask1)
		# Apply the mask to the data
		for dou in range (0,2):
			data[:,dou] = np.ma.masked_where(np.ma.getmask(datamask), data[:,dou])
	
		# save data in the folder Data/
		# Be carefull, there is no header...	
		print(u'   Saving the data in Data/ ...')
		header = 'Dist_(m) Alt_(m) Alt_freq'
		np.savetxt('Data/data_' + title + '_' + str(iii) + '.txt',data, header = header)
		np.savetxt('Data/datamask_' + title + '_' + str(iii) + '.txt',datamask)
		np.savetxt('Data/falti_' + title + '_' + str(iii) + '.txt',falti)
		header = 'Dist_(m) Min_Alt_(m) Max_Alt_(m)  Median_Alt_(m)  Mean_Alt_(m)'
		np.savetxt('Data/statslines_' + title + '_' + str(iii) + '.txt',statslines, header = header)
		
		# Plot the graph
		plot_graph(data = data, 
	    	       datamask = datamask, 
		           statslines = statslines, 
		           title = title, 
	    	       xdist = xdist,
	    	       xstep = xsteps,
		           boxwidth = boxwidth, 
		           factor = factor,
		           iii = iii,
		           corrnan = corrnan,
		           sizeplotx = sizeplotx,
		           sizeploty = sizeploty,
		           densitymin = densitymin,
		           densitymax = densitymax,
		           ylimmin = ylimmin,
		           ylimmax = ylimmax,
	    	       synthetic = synthetic)
	# Clean the workspace
	clean()
	
	return

############################################################################

def clean():
	"""
	Function to clean the workspace after the plot of each profile
	It removes the temporary shapefiles from the TMP/ directory
	"""	
	# cleaning
	# erase the temporary shapefiles
	print(u'   Cleaning....')
	files = os.listdir("TMP")
	for f in files:
		full_path = os.path.join("TMP",f)
		if not os.path.isdir(full_path):
			os.remove(full_path) 
	os.rmdir("TMP")
	os.mkdir("TMP")
	print(' ')
	return




######!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import  division
# This to be sure that the result of the division of 2 integers is a real, not an integer
from __future__ import absolute_import
from __future__ import print_function

__version__ = "2.0.2"

# Import modules
import sys
import os
#import copy
import numpy as np

# Import all the functions
__all__ = ['raster_tools', 'profiles', 'swath', 'plotgraph', 'checks']

from .raster_tools import *
from .profiles import *
from .swath import *
from .plotgraph import *
from .checks import *



######!/usr/bin/env python
# -*- coding: utf-8 -*-

# Do divisions with Reals, not with integers
# Must be at the beginning of the file
from __future__ import division
from __future__ import absolute_import
from __future__ import print_function

## Import Python modules
## I have problems to install rasterio : it does not find gdal libraries... from kingchaos
modulesNames = ['sys', 'math', 'os', 'utm', 'rasterstats', 'shapely']
for module in modulesNames:
	try:
		# because we want to import using a variable, do it this way
		module_obj = __import__(module)
		# create a global object containging our module
		globals()[module] = module_obj
	except ImportError:
		#sys.exit(u"ERROR : Module " + module + " not present. \n\n Please, install it \
		raise ImportError(u"ERROR : Module " + module + " not present. \n\n Please, install it \
			      \n\n Edit the source code for more information")
from os import path, access, R_OK, mkdir         # W_OK for write permission.
from rasterstats import raster_stats, zonal_stats             # For stats on rasters
from shapely.geometry import Polygon, LineString, shape, box, MultiPolygon
from shapely import affinity
from shapely import speedups
if speedups.available:
	speedups.enable()
try:
	import numpy as np                               # need version 1.7 or higher
except ImportError:
	#sys.exit(u"ERROR : Module Numpy not present. \n\n Please, install it \
	raise ImportError(u"ERROR : Module Numpy not present. \n\n Please, install it \
		      \n\n Edit the source code for more information")
try:
	from osgeo import gdal, gdalnumeric, ogr, osr    # For GIS operations
	from osgeo.gdalconst import *
except ImportError:
	#sys.exit(u"ERROR : Module osgeo/gdal not present. \n\n Please, install it \
	raise ImportError(u"ERROR : Module osgeo/gdal not present. \n\n Please, install it \
		      \n\n Edit the source code for more information")
###############################################################################

def calc_length(A,B):       
	"""
	function to calcule the distance
	between two points A and B given by their projected coordinates (in m),   
	The result is in meters !
	
	INPUTS:
	   A = Coordinates of the point A
	   B = Coordinates of the point B
	
	OUTPUTS:
	   calc_lengthm = length between A and B in m
	
	USAGE:
	   calc_lengthm = calc_length(A,B)
	"""
	
	calc_lengthm = np.sqrt((A[0] - B[0]) ** 2 + (A[1] - B[1]) ** 2)
	
	return calc_lengthm

############################################################################
       
def bearing(XA,YA,XB,YB):       
	"""
	Function to calcule the bearing on a sphere
	between two points A and B given by their long/lat coordinates
	
	INPUTS:
	   XA, YA = A coordinates in lat-long
	   XB, YB = B coordinates in lat-long
	
	OUTPUTS:
	   angle = bearing from A to B
	
	USAGE:
	   angle = bearing(XA,YA,XB,YB)
	   
	Written by X. Robert, june 2014
    """   
	# Not used anymore
	
	R = 6371  # Rayon de la terre en km
	
	# Convert in Radians
	lon1, lat1, lon2, lat2 = map(math.radians, [XA, YA, XB, YB])
	
	dLon = lon2 - lon1
	y = np.sin(dLon) * np.cos(lat2)
	x = np.cos(lat1) * np.sin(lat2) \
		- np.sin(lat1) * np.cos(lat2) * np.cos(dLon)
	
	return math.degrees(math.atan2(y, x))

############################################################################

def project_points(A, B, srs, A_utm, B_utm, test_N, iii, ggg):
	"""
	Function to project A and B from Lat-Long to UTM coordinates
	
	INPUTS:
	   A, B = the two points to project in utm
	   srs = Coordinate system of A & B
	   A_utm, B_utm = the points where the value of the projection will be recorded
	   test_N = Boolean that tells if the points are north or south
	   iii = iteration number (rank in the A list)
	   ggg = iteration number (rank in the B list)
	
	OUTPUTS:
	   A_utm, B_utm = Output projected coordinates
	   srs_new = new projection system
	
	USAGE:
	   A_utm, B_utm, srs_new = project_points(A, B, A_utm, B_utm, test_N, iii, ggg)
	
	Written by X. Robert, june 2015
	"""
	
	# Check if the points of the profile are in the same UTM zone
	if utm.from_latlon(A[iii ,1], A[iii ,0])[2] != utm.from_latlon(B[ggg, 1], B[ggg, 0])[2] \
	    or utm.from_latlon(A[0, 1], A[0, 0])[2] != utm.from_latlon(A[ggg,1], A[ggg, 0])[2]:
		# If not, Raise Warning, and choose only one UTM zone
		if utm.from_latlon(A[iii ,1], A[iii ,0])[0] < utm.from_latlon(B[iii, 1], B[iii, 0])[0]:
			utm_zone = utm.from_latlon(A[0,1], A[0,0])[2]
		else:
			utm_zone = utm.from_latlon(B[0,1], B[0,0])[2]
		if iii == 0:
			print(u'WARNING : The profile is over two UTM zones ! \n'
			      u'          I am choosing one of them : UTM%i' % utm_zone)	
			print(u'          If you get this warning, you may check if on a GIS software if')
			print(u'          the computed profile is at the right place on the DEM.')
			print(u'          Do not be surprised if you get an error in the calcul of statistics,')
			print(u'          that means that your projected profile is out of the bounds of the projected DEM.')
			print(u'          To solve the problem, use as entries :')
			print(u'               - The projected DEM,')
			print(u'               - Coordinates of points that you extract on the projected DEM!')
		# Define the new spatial reference
		srs_new = osr.SpatialReference()
		# Set the projection to UTM with the zone that correspond to the datapoints
		srs_new.SetUTM(utm_zone, test_N)
		transformxx = osr.CoordinateTransformation(srs, srs_new)
		# Find coordinates of the new raster
		(A_utm [iii, 0], A_utm [iii, 1], junk) = transformxx.TransformPoint(A[iii,0], A[iii,1])
		(B_utm [ggg, 0], B_utm [ggg, 1], junk) = transformxx.TransformPoint(B[ggg,0], B[ggg,1])
	else:
		# If in the same UTM zone, calculate the new coordinates in UTM
		A_utm [iii, 0] = utm.from_latlon(A[iii,1], A[iii, 0])[0]
		A_utm [iii, 1] = utm.from_latlon(A[iii,1], A[iii, 0])[1]
		B_utm [ggg, 0] = utm.from_latlon(B[ggg,1], B[ggg, 0])[0]
		B_utm [ggg, 1] = utm.from_latlon(B[ggg,1], B[ggg, 0])[1] 		
		# Get the UTM zone
		utm_zone = utm.from_latlon(A[0,1], A[0,0])[2]
		# Define the new spatial reference
		srs_new = osr.SpatialReference()
		# Set the projection to UTM with the zone that correspond to the datapoints
		srs_new.SetUTM(utm_zone, test_N)
	
	print(u'Projection to UTM zone %i' % utm_zone)
	
	#return A_utm[iii, 0], A_utm[iii, 1], B_utm[iii, 0], B_utm[iii, 1], srs_new
	return A_utm, B_utm, srs_new

############################################################################

def raster_projection(rasterfnme, rasterdata, src_geotrans,  srs, dst_filename, srs_new, kmdeg, mem = 'NULL'):
	"""
	Project raster. Return a memory raster if mem != 'NULL' or a tiff file if mem == 'NULL'
	
	INPUTS:
	   rasterfnme = input raster name
	   rasterdata = input raster data
	   src_geotrans = Coordinate transform vector
	   srs = input coordinate system
	   dst_filename = output raster name
	   srs_new = output coordinate system into to project the raster
	   kmdeg = factor to convert from degrees to km
	   mem = tells if the output raster should be stored in a file ( = 'NULL')
	          or in memory (= 'MEMORY')
	
	OUTPUTS:
	   if mem = 'NULL', no variable outpût, only a projected raster is recorded in a new file
	   if mem = 'MEMORY',
	          dst = Raster reprojected in Lat Long, stored in memory
	          out_srs  = coordinate system
	
	USAGE:
	   to project a raster in a new tiff file:
	       raster_projection(rasterfnme, rasterdata, srs, dst_filename, srs_new, mem = 'NULL')
	   to project a raster and to store it in Memory:
	        dst_tmp = raster_projection(rasterfnme, 
		                                rasterdata, 
		                                srs, 
		                                dst_filename,  
		                                latlong_srs, 
		                                mem = 'MEMORY')
	   
	Written by X. Robert, june 2015
	"""
	
	if mem == 'NULL':
		# Test if the input raster is in lat-long
		if int(srs.GetAttrValue("AUTHORITY", 1)) != 4326:
			raise NameError(u'ERROR : the input raster for the raster_projection function '
			         u'is not in geographic coordinates (i.e. EPSG != 4326)')
		# Test if raster already exists
		if path.isfile(dst_filename):
			# If yes, do not do the projection, skip it
			print(u'The raster %s has already be projected, I will not do it another time, '
			      u'I am thus using %s' % (rasterfnme, dst_filename))
		else:
			#print rasterdata.GetMetadata()
			#print rasterdata.GetRasterBand(1).GetStatistics(True, True)
			# Define the new raster
			#band = rasterdata.GetRasterBand(1)
			dst = gdal.GetDriverByName('GTiff').Create(dst_filename,
													   rasterdata.RasterXSize,
													   rasterdata.RasterYSize,
													   1,
													   GDT_Float32)
			#    	                                   band.DataType)
			# Set up the transform (from http://pangea.stanford.edu/~samuelj/musings/dems-in-python-pt-2-projecting-dems.html)
			# Create the coordinate transform object
			transformxx = osr.CoordinateTransformation(srs, srs_new)
			# Find coordinates of the new raster
			(ulx, uly, ulz) = transformxx.TransformPoint(src_geotrans[0], src_geotrans[3])
			(lrx, lry, lrz) = transformxx.TransformPoint(src_geotrans[0] + src_geotrans[1] * rasterdata.RasterXSize,
														 src_geotrans[3] + src_geotrans[5] * rasterdata.RasterYSize)
			# Set the new geotransform
			geotransform_proj = (ulx, src_geotrans[1] * 1000 / kmdeg, src_geotrans[2], uly, src_geotrans[4], -src_geotrans[1] * 1000 / kmdeg)
			# Set the new metadata
			dst.SetMetadata(rasterdata.GetMetadata())
			# Give the same extension to the new raster (REM : is it good ????)
			dst.SetGeoTransform(geotransform_proj)
			# Apply the projection		
			dst.SetProjection(srs_new.ExportToWkt())		
			# Do the reprojection
			gdal.ReprojectImage(rasterdata, # InRaster
								dst,        # OutRaster
								srs.ExportToWkt(), # src_wkt
								srs_new.ExportToWkt(), # dst_wkt
								#gdal.GRA_CubicSpline)
								#GRA_CubicSpline)
								#GRA_NearestNeighbour)
								GRA_Bilinear)    # eResampleAlg		
			# clean the memory
			dst = None
		return
	else:
		# Define the new raster in Memory
		dst = gdal.GetDriverByName('MEM').Create('',
												rasterdata.RasterXSize,
												rasterdata.RasterYSize,
												1,
												GDT_Float32)
		# Set up the transform (from http://pangea.stanford.edu/~samuelj/musings/dems-in-python-pt-2-projecting-dems.html)
		# Create the coordinate transform object
		transformxx = osr.CoordinateTransformation(srs, srs_new)
		# Find coordinates of the new raster
		(ulx, uly, ulz) = transformxx.TransformPoint(src_geotrans[0], src_geotrans[3])
		(lrx, lry, lrz) = transformxx.TransformPoint(src_geotrans[0] + src_geotrans[1] * rasterdata.RasterXSize,
													 src_geotrans[3] + src_geotrans[5] * rasterdata.RasterYSize)
		# Set the new geotransform
		geotransform_proj = (ulx, src_geotrans[1] * 1000 / kmdeg, src_geotrans[2], uly, src_geotrans[4], -src_geotrans[1] * 1000 / kmdeg)
		# Set the new metadata
		dst.SetMetadata(rasterdata.GetMetadata())
		# Give the same extension to the new raster (REM : is it good ????)
		dst.SetGeoTransform(geotransform_proj)
		# Apply the projection		
		dst.SetProjection(srs_new.ExportToWkt())		
		# Do the reprojection
		gdal.ReprojectImage(rasterdata, # InRaster
							dst,        # OutRaster
							src.ExportToWkt(), # src_wkt
							srs_new.ExportToWkt(), # dst_wkt
							GRA_Bilinear)    # eResampleAlg		
		return dst
			
############################################################################

def project_raster(rasterfnme, rasterdata, src_geotrans, srs, A, B, Coord, xstep, boxwidth):
	"""
	Function to project the raster and the lines
	
 	INPUT:
 		- rasterfnme : name of the raster to project (with the extension)
 		- rasterdata : 
 		- src_geotrans : data Geotransform of the raster to project
 		- srs : coordinate system of the original raster
 		- A and B : Coordinates that define the different lines
 		- Coord : Set the coordinate system in which are given A and B
 		- xstep = size of the x stepping in the original raster metric
 		- boxwidth : y-width in the original raster metric of the box
 	
 	OUTPUT:
 		- xstep = size of the x stepping in the projected raster metric
 		- boxwidth = y-width in the projected raster metric of the box
 		- Coord = new coordinate system
 		- srs = new projected system
 		- dst_filename
 		- a_utm, b_utm = Coordinates of the points A and B transformed in UTM
 	
 	USAGE:
 	    xstep, boxwidth, Coord, srs, dst_filename, a_utm, b_utm =
 	       project_raster(rasterfnme, rasterdata, src_geotrans, srs, A, B, Coord, xstep, boxwidth)
 	
 	Written by X. Robert, June 2015
 	"""
 	
	A = np.array(A)
	B = np.array(B)
	A_utm = np.zeros(A.shape)
	B_utm = np.zeros(B.shape)
	
	# find new projection
	# If data in Lat-long:
	# Calcul the factor to convert from degrees to km
	R = 6378
	kmdeg = 180 /(math.pi * R * np.cos(np.radians(A[0,1])))
	if srs.GetAttrValue('projcs') == None:
	#if Coord == 'latlong':# or srs.GetAttrValue('projcs') == None:
		# Check if north or South
		if A[0,1] < 0:
			test_N = 0
		else:
			test_N = 1
		# Do the lat-long to UTM conversion
		for iii in range (0, A.shape[0]):
			A_utm, B_utm, srs_new = project_points(A, B, srs, A_utm, B_utm, test_N, iii, iii)
			
		# Set the output
		# New raster file name
		dst_filename = rasterfnme[0:-4] + '_projUTM.tif'
		
		raster_projection(rasterfnme, rasterdata, src_geotrans, srs, dst_filename, srs_new, kmdeg, mem = 'NULL')
		
	else:
		# set the projection reference for lat long
		latlong_srs = osr.SpatialReference()
		latlong_srs.ImportFromEPSG(4326)
		dst_filename = ''
		# convert to lat-long (in memory)
		dst_tmp = raster_projection(rasterfnme, 
		                            rasterdata,
		                            src_geotrans, 
		                            srs, 
		                            dst_filename,  
		                            latlong_srs,
		                            kmdeg, 
		                            mem = 'MEMORY')
		# Project A and B
		A_lat = np.zeros(A.shape)
		B_lat = np.zeros(B.shape)
		transformxx = osr.CoordinateTransformation(srs, latlong_srs)
		for iii in range (0, A.shape[0]):
			# Find coordinates of the new raster
			(A_lat [iii, 0], A_lat [iii, 1], junk) = transformxx.TransformPoint(A[iii,0], A[iii,1])
			(B_lat [iii, 0], B_lat [iii, 1], junk) = transformxx.TransformPoint(B[iii,0], B[iii,1])
	    # find if in northern or southern hemisphere
		if A_lat[0,1] < 0:
			test_N = 0
		else:
			test_N = 1
		for iii in range (0, A.shape[0]):
			A_utm, B_utm, srs_new = project_points(A_lat, B_lat, srs, A_utm, B_utm, test_N, iii, iii)
	    # Set the output
		# New raster file name
		dst_filename = rasterfnme[0:-4] + '_projUTM.tif'
		# Project the lat-long memory raster to UTM
		raster_projection(rasterfnme, dst_tmp, src_geotrans, srs, dst_filename, srs_new, kmdeg, mem = 'NULL')
		dst_tmp = None
	
	# change to numpy array
	a_utm = np.array(A_utm)
	b_utm = np.array(B_utm)	
	# Convert the parameters to meters
	xstep[:] = [zz * 1000 / kmdeg for zz in xstep]
	boxwidth[:] = [zz * 1000 / kmdeg for zz in boxwidth]
	# Change the coordinates system variable
	srs = srs_new
	# Change the coordinates description
	Coord = 'utm'
	
	return xstep, boxwidth, Coord, srs, dst_filename, a_utm, b_utm, test_N

############################################################################
def CopyBand(srcband, dstband):
	"""
	from http://www.mit.edu/afs.new/athena/software/fwtools/current/FWTools-linux-i686-3.1.0/usr/bin/gdal_fillnodata.py
	"""
	for line in range(srcband.YSize):
		line_data = srcband.ReadRaster( 0, line, srcband.XSize, 1 )
		dstband.WriteRaster( 0, line, srcband.XSize, 1, line_data,
		                     buf_type = srcband.DataType )

############################################################################

def removeNoData(inraster, dst_filename = 'None', mask = 'default', NoData = 'None'):
	"""
	function to remove NoData values from a raster
	
	Modification of http://www.mit.edu/afs.new/athena/software/fwtools/current/FWTools-linux-i686-3.1.0/usr/bin/gdal_fillnodata.py
	
	INPUTS:
	   inraster = raster data to clean
	   dst_filename = name of the raster to save
	   mask = see ogr.FillNodata.py
	   NoData = Value of the NoData. 
	            If 'None', I consider that the NoData value is under or equal to 0.0
	
	OUTPUTS:
		newraster = cleaned raster data
		
	NEEDS strong improvements......
	"""
	
	print(u'Cleanning the DEM, removing holes...')
	
	# Options for gdal
	max_distance = 100
	smoothing_iterations = 0
	options = []
	quiet_flag = 0
	#src_filename = None
	src_band = 1

	# 	Verify we have next gen bindings with the sievefilter method.
	try:
		gdal.FillNodata
	except:
		print(u'')
		print(u'gdal.FillNodata() not available.  You are likely using "old gen"')
		print(u'bindings or an older version of the next gen bindings.')
		print(u'')
		raise ModuleError(u"ERROR : Problem with gdal.FillNodata.") 
		#sys.exit(1)
		
	srcband = inraster.GetRasterBand(src_band)
	
	if mask is 'default':
		maskband = srcband.GetMaskBand()
	elif mask is 'none':
		maskband = None
	else:
		mask_ds = gdal.Open( mask )
		maskband = mask_ds.GetRasterBand(1)
	
	#   Create output file if one is specified.
	if dst_filename is not None:
		
		drv = gdal.GetDriverByName('GTiff')
		dst_ds = drv.Create( dst_filename,inraster.RasterXSize, inraster.RasterYSize,1,
		                 srcband.DataType )
		wkt = inraster.GetProjection()
		if wkt != '':
			inraster.SetProjection( wkt )
		inraster.SetGeoTransform( inraster.GetGeoTransform() )
		
		dstband = inraster.GetRasterBand(1)
		CopyBand( srcband, dstband )	
	else:
		dstband = srcband
	
	#	Invoke algorithm.
	if quiet_flag:
		prog_func = None
	else:
		prog_func = gdal.TermProgress
	
	result = gdal.FillNodata(dstband, 
	                         maskband,
                             max_distance, 
                             smoothing_iterations, 
                             options,
                             callback = prog_func )
	# close rasters
	src_ds = None
	dst_ds = None
	mask_ds = None
	
	newraster = gdal.Open(dst_filename, gdal.GA_ReadOnly)
	
	return newraster









######!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
##########################################################
#                                                        #  
#          Slope profile extraction script               #
#                                                        #  
#                 By Xavier Robert                       #
#           From Peter Van der Beek codes                #
#                Montreal, 09/10/18                      #
#                  Lima, june 2014                       #
#                  Lima, june 2015                       #
#                                                        #  
##########################################################

Written by Xavier Robert, june 2015

xavier.robert@univ-grenoble-alpes.fr

(c) licence CCby-nc : http://creativecommons.org/licenses/by-nc/3.0/ 2015

"""

# Do divisions with Reals, not with integers
# Must be at the beginning of the file
from __future__ import division
from __future__ import absolute_import
from __future__ import print_function

# Import Python modules
# I have problems to install rasterio : it does not find gdal libraries... from kingchaos
#modulesNames = ['sys', 'math', 'os', 'utm', 'warnings', 'rasterstats', 'shapely', 'copy', 'time', 'rasterio']
modulesNames = ['sys', 'time', 'copy']
for module in modulesNames:
	try:
		# because we want to import using a variable, do it this way
		module_obj = __import__(module)
		# create a global object containging our module
		globals()[module] = module_obj
	except ImportError:
		#sys.exit(u"ERROR : Module " + module + " not present. \n\n Please, install it \
		raise ImportError(u"ERROR : Module " + module + " not present. \n\n Please, install it \
			      \n\n Edit the source code for more information")
try:
	import numpy as np                               # need version 1.7 or higher
except ImportError:
	#sys.exit(u"ERROR : Module Numpy not present. \n\n Please, install it \
	raise ImportError(u"ERROR : Module Numpy not present. \n\n Please, install it \
		      \n\n Edit the source code for more information")
try:
	from osgeo import gdal, gdalnumeric, ogr, osr    # For GIS operations
	from osgeo.gdalconst import *
except ImportError:
	#sys.exit(u"ERROR : Module osgeo/gdal not present. \n\n Please, install it \
	raise ImportError(u"ERROR : Module osgeo/gdal not present. \n\n Please, install it \
		      \n\n Edit the source code for more information")
try:
	from progress.bar import Bar                     # For a progress bar in the terminal output
except ImportError:
	#sys.exit(u"ERROR : Module progress not present. \n\n Please, install it \
	raise ImportError(u"ERROR : Module progress not present. \n\n Please, install it \
		      \n\n Edit the source code for more information")
from distutils.version import LooseVersion, StrictVersion

from .raster_tools import *
from .profiles import *
from .plotgraph import *
from .checks import *

###############################################################################

def swathp(rasterfnme = None, A = None, B = None, Coord = 'utm', factor = 1000,
			xsteps = [5000], boxwidths = [20000], binsize = 20, title = 'Swath_profile',
			shpbox = 'shpbox.shp', sizeplotx = None, sizeploty = None,
			densitymin = None, densitymax = None, ylimmin = None, ylimmax = None,
			remNoData = False, corrnan = False, nodatav = 0.0,
			multipoints = [False], nbpointsint = [0], pointsdic = None, printpointsi = False,
			C = None, D = None, E = None, F = None, G = None, H = None, I = None, J = None,
			idensity = False):
	"""
	
	This code is to built topographic swath profile through a region, 
	with a plot of the density of elevation in the box defined around the profile.
	
	It uses a DEM that is projected in UTM (best input), 
	but will also work with geographic coordinates (lat-long).
	
	USAGE
	Inside a (i)python environnement:
	To import the module:
		>>> from swath import swathp
	To plot a swath profile [A,B] through the raster 'dem.tif':
		>>> swathp()

	INPUTS
	To use options or inputs, you need to set them as
	swath(option_name = option_value, [...])
	
	1. rasterfnme: name of the raster to work with
					Add the full path to the raster. Personally, I like to store my rasters in a DEM/folder
					Be aware that if the input raster is not projected in UTM, the code will create a projected raster in the same folder.
					ex: rasterfnme = 'Dem/Dem_Fusion-Peru_projUTM.tif'
					Default = None
	2. A, B: Coordinates of the 2 points A and B defining the whole profile in the same projection system than the DEM
				If multipoints profile (see further), these two points define the two extreme points.
				Be careful, do no take oceans, otherwise, statistics will be biased
				If different profiles:
				ex: A = [(-78.255,-9.713),(,),...]
					B = [(-77.255,-9.218),(,),...]
				each column corresponding to one different profile
				Default: A = None, B = None
	3. Coord: Units of the points coordinates. 
			- If Lat/Long : Coord = 'latlong'
			- If UTM : Coord = 'utmZONE'
			- If the dem is a synthetic dem (This is to avoid the problem of projections
			  If it is set to True, change the value of Factor if needed!) : Coord = 'synthetic'
			- If else, give a projection name that is NOT 'latlong' or 'utmZONE' or 'synthetic'
			ex: Coord = 'utm'
				Coord = 'latlong'
				Coord = 'synthetic'
			Default = 'utm'
	4. factor: Factor is to convert to km (generaly 1000)
				if the unit of the DEM is 'meters', factor = 1000
				if the unit of the DEM is 'kilometers', factor = 1
				if the DEM is synthetic, units are arbitrary, so it could be factor = 0.001 (Test it !)
				Default: factor = 1000
				(ex: factor = 0.01)
	5. xsteps: Stepping along the profile in the same projection/coordinates system than the DEM
				If more than one profile with different profiles: xsteps = [5000, 2000,...], each column corresponding to one different profile
				If all the profiles have the same xsteps, just use one column 
				ex: xsteps = [5000]
				Default = [5000]
	6. boxwidths: with of the box around the profile from where are extracted the stats in the same units than the DEM (m if m; km if km; deg if deg)
				if several profiles with different profiles: boxwidths = [20000, 15000,...], each column corresponding to one different profile
				If all the profiles have the same boxwidth, just use one column 
				ex: boxwidths = [20000]
				Default = [20000]
	7. binsize: altitude binsize (for the altitude frequency plot) in the same units than the DEM (m if m; km if km; deg if deg)
				ex: binsize = 20
				Default = 20
	8. title: title of the graphic
			The name will also be used to define the name:
				- in which the shapefiles are stored
				- of the output files
			ex: title = 'Synth-Essai'
			Default = 'Swath_profile'
	9. shpbox: Name of the shapefile in which we extract the profile
			Default: shpbox = 'shpbox.shp'
	10. sizeplotx, sizeploty: size of the plot.
							Standard size is sizeplotx = 8 and sizeploty = 6
							If you want to use the default/automatic setting, just give the value None to the variables
							Default: sizeplotx = None
									 sizeploty = None
	11. densitymin, densitymax: set the density colorbar limits (between [0,1]).
								Set it to None, if you want to keep the automatic settings
								Default: densitymin = None
										 densitymax = None
	12. remNoData: Flag to remove (True) or not (False) the NoData values from a DEM
					2015/08 : does not work very well, avoid it for the moment.
					Default: remNoData = False
	13. corrnan: Flag to correct (True) or no (False) the graph from the Nan values
				Be careful, it replace the NaN values with the min value of the frequency
				Default: corrnan = False
	14. nodatav: value of the NoData
				Default: nodatav = 0.0
	15. multipoints: Multipoints section : 
					Flag to set a profil with multipoints
					[False] = only two points (Default)
					[True] = more than two points
					[True, False,...] if several  transects
					ex: multipoints = [False]
	16. nbpointsint: Multipoints section :
					number of intermediary points in the profile
					I choose to limit to 9 intermediary points.
					If different profiles : nbpointsint = [...,3,2,1]
					Choose the order of the profiles with a decreasing number of intermediary points to avoid error in the code
					ex: nbpointsint = [0]
	17. pointsdic: Multipoints section :
					dictionnary to assign a number to the different points. It should contain the same number of lines than the number of points
				ex: pointsdic = {1 : C,
								2 : D,
								3 : E,
								4 : F,
								... : ...
								}
	18. printpointsi: Multipoints section :
					Flag to print (True) or not (False) the position of the intermediary points on the profile
					ex: printpointsi = True
					Default = False
	19. C,D,...,J: Multipoints section :
				intermediary points in the profile, given from A to B
				Be aware of the order !
				I choosed to limit to 9 intermediary points C,D,E,F,G,H,I,J
				Give the name C for the 1st intermediary point (C = [(-78.255,-9.713),(,),...])
							  D for the 2nd intermediary point (D = [(-78.255,-9.713),(,),...])
							  E for the 3rd intermediary point
							  ...
	20. idensity: Flag to plot the density (True) or not (Default, False)
		
	OUPUTS
	Inside the working directory, the code build several folders :
		- Data/: For each profile, the code outputs XXXXX files in Data/:
			+ data_title_Nbprofile.txt:
			+ datamask_title_Nbprofile.txt:
			+ falti_title_Nbprofile.txt:
			+ statslines_title_Nbprofile.txt:	
		- Graphs/: for each profile, the code outputs here the graphs in pdf
		- shpbox/ (defined in the Variable declaration): In this directory, for each profile, the code outputs:
			+ a shapefile defining the line between the two points of the profile
	    	+ a shapefile the define the box in which the transect is extracted

	CONTACT
		If needed, do not hesitate to contact the author. 
		Please, use https://isterre.fr/spip.php?page=contact&id_auteur=303

	LICENCE
	This package is licenced with `CCby-nc` (https://creativecommons.org/licenses/by-nc/2.0/)
	"""
	start_time = time.time()
	
	print(' ')
	print(' ')
	print('******************************************')
	print('Program to extract swath profiles from DEM')
	print('     Written by X. Robert, ISTerre')
	print('           June 2014-2015      ')
	print('******************************************')
	
	# Check the version of the module Rasterstats : if Rasterstats version is strickly under 0.10, this script crashes
	# This is because the structure of the miniarray changed from the version 0.12
	if StrictVersion(rasterstats.__version__)<StrictVersion('0.11'):
		raise ModuleError(u'ERROR: the module Rasterstats is too old. --> UPDATE to a version >= 0.12')
	
	# If the minima input data are not given print the help file
	if rasterfnme == None or A == None or B == None: help(swathp)
	
	# Check the files... 
	if Coord == 'synthetic': 
		synthetic = True
	else:
		synthetic = False
	xsteps, boxwidths, Coord, srs_init, srs, dst_filename, a, b, a_utm, b_utm, test_N, shpbox, ulx, lrx, lry, uly, projdone = checkfiles(rasterfnme, 
						A, B, xsteps, boxwidths, shpbox, title, 
						Coord, synthetic,
						multipoints, remNoData)
	
	print(u'  ')
	print(u'Number of profiles : ' + str(a.shape[0]))
	print(u'With xsteps = %s and boxwidths = %s' % (xsteps, boxwidths)) 
	
	# Begin the stepping on the number of profiles (a.shape[0])
	for iii in range (0, a.shape[0]):
		print(u' ')
		print (u'Doing swath profile ' + str(iii + 1) + u'/' + str(a.shape[0]))
		if len(xsteps) != 1:
			xstep = xsteps[iii]
		else:
			xstep = xsteps[0]
		if len(boxwidths) != 1:
			boxwidth = boxwidths[iii]
		else:
			boxwidth = boxwidths[0]
		aa = (a_utm[iii,0], a_utm[iii,1])
		bb = (b_utm[iii,0], b_utm[iii,1])
		
		multip = False
		if len(multipoints) != 1:
			if multipoints[iii]:
				ggg = -1
				for indexg in range (0,iii+1):
					if multipoints[indexg]:
						ggg += 1
			multip = multipoints[iii]
		else:
			ggg = 0
			multip = multipoints[0]

		if multip:
			print(u'   profile with %i intermediary points' % nbpointsint[ggg])
			# initiate the first point with A
			aa1 = aa
			A = np.array(A)
			xdisttot = 0
			if printpointsi:
				xpoint = []
			#cc = np.zeros((nbpointsint[iii], 2))
			for kkk in range (0, nbpointsint[ggg]+1):
				D_utm = np.zeros(A.shape)
				if kkk != nbpointsint[ggg]:
					# if this is not the end of the profile, set the second point to cc[kkk]											
					c = pointsdic[kkk+1]
					c = eval(c)					
					c = np.array(c)					
					#ggg = -1
					#for indexg in range (0,iii+1):
					#	if multipoints[indexg]:
					#		ggg += 1
					
					# check if the point is in the profile
					if  not (ulx < c[ggg][0]< lrx):
						raise NameError(u"ERROR : Intermediary point%s is outside the DEM border %s" % (c, (ulx, lrx)))
					if  not (lry < c[ggg][1]< uly):
						raise NameError(u"ERROR : Intermediary point%s is outside the DEM border %s" % (c, (lry, uly)))
					
					if projdone:
						C2_utm = np.zeros(c.shape)
						junk = D_utm
						# do the projection if needed
						junk, c, junk2 = project_points(A, c, srs_init, D_utm, C2_utm, test_N, iii, ggg)
						
					bb1 = c[ggg]
				else:
					# if this is the end of the profile, set the second point to B
					bb1 = bb
					
				# call the main code for each sub-profile
				if kkk == 0:
					shpbox = shpbox[0:-4] + '_sub_' + str(kkk+1) + '.shp'
				else:
					shpbox = shpbox[0:-5]+ str(kkk+1) + '.shp'
				time_st = time.time()
				statslines, data, xdist = main_xa(A = aa1, B = bb1,
			                                      xsteps = xstep, 
			                                      boxwidth = boxwidth, 
			                                      binsize = binsize, 
			                                      title = title,
			                                      shpbox = shpbox, 
			                                      rasterfnme = dst_filename, 
			                                      srs = srs, 
			                                      factor = factor,
			                                      corrnan = corrnan,
			                                      synthetic = synthetic, 
			                                      iii = iii,
			                                      multipoints = multip,
			                                      nbpointsint = nbpointsint[ggg],
			                                      kkk = kkk)
				# update the first point of the sub-profile
				aa1 = c[ggg]
				xdisttot = xdisttot + xdist
				# create backup files
				if kkk == 0:
					statslinestot = copy.copy(statslines)
					datatot = copy.copy(data)
				else:
					print(u'   Concatenating profiles... \n'
					      u'       It can be long... Be patient')
					try: 
						statslinestotbackup = copy.copy(statslinestot)
						del statslinestot
					except NameError:
						raise NameError(u'ERROR: Statslinetot does not exists...')
					try:
						datatotbackup = datatot
						del datatot
					except NameError:
						raise NameError(u'ERROR: datatot does not exists...')	
					statslinestot = np.empty((statslinestotbackup.shape[0] + statslines.shape[0],5))
					datatot = np.empty((datatotbackup.shape[0] + data.shape[0]-1,3))
					if printpointsi:
						xpoint.append(max(statslinestotbackup[:,0]))
					# Stacking the files
					statslinestot = np.concatenate((statslinestotbackup, statslines), axis = 0)
					for lll in range (statslinestotbackup.shape[0],statslinestot.shape[0]):
						statslinestot[lll,0] = max(statslinestotbackup[:,0]) \
							                     + statslines[lll - statslinestotbackup.shape[0],0]
					datatot = np.concatenate((datatotbackup, data), axis = 0)
					
					# Define the progress-bar that is output in the terminal window
					bar = Bar(u'       Processing concatenation %i.%i' % ((iii + 1), kkk), 
					           max = datatot.shape[0]-datatotbackup.shape[0] + 1, 
					           suffix = '%(index)d / %(max)d ')
					for lll in range (datatotbackup.shape[0], datatot.shape[0]):
						# This is this loop that is long...
						datatot[lll,0] = max(datatotbackup[:,0]) + data[lll - datatotbackup.shape[0],0]
						bar.next()
					bar.next()
					# Finish the progress-bar
					bar.finish()
					time_int = time.time() - time_st
					print(u'       This subprofil needed %i seconds to run' % time_int)
				# Clean the workspace
				clean()
			# mask the data where it is out of the bounds statslines[:,1] and statslines[:,2]
			# mask the data where it is null (set null value to nan)
			datamask1 = np.ma.masked_where(datatot[:,2] == 0, datatot[:,2])
			# mask the data where it is closed to 1 to get a colorbar not overprinted by only 1 value
			datamask = np.ma.masked_where(datamask1 == 1, datamask1)
	
			# save data in the folder Data/
			# Be carefull, there is no header...	
			print(u'   Saving the data in Data/ ...')
			np.savetxt('Data/data_' + title + '_' + str(iii) + '.txt',datatot)
			np.savetxt('Data/datamask_' + title + '_' + str(iii) + '.txt',datamask)
			#np.savetxt('Data/falti_' + title + '_' + str(iii) + '.txt',falti)
			np.savetxt('Data/statslines_' + title + '_' + str(iii) + '.txt',statslinestot)
			
			# Plot the graph
			plot_graph(data = datatot, 
				       datamask = datamask, 
		    	       statslines = statslinestot, 
			           title = title, 
		    	       xdist = xdisttot,
		    	       xstep = xstep,
			           boxwidth = boxwidth, 
				       factor = factor,
				       iii = iii,
				       corrnan = corrnan,
				       sizeplotx = sizeplotx,
				       sizeploty = sizeploty,
				       densitymin = densitymin,
				       densitymax = densitymax,
				       ylimmin = ylimmin,
				       ylimmax = ylimmax,
				       synthetic = synthetic,
				       xpoint = xpoint)	
		else:
			# call the main code
			main_xa(A = aa, B = bb,
			        xsteps = xstep, 
			        boxwidth = boxwidth, 
			        binsize = binsize, 
			        title = title, 
			        shpbox = shpbox, 
			        rasterfnme = dst_filename, 
			        srs = srs, 
			        factor = factor,
			        sizeplotx = sizeplotx,
			        sizeploty = sizeploty,
			        densitymin = densitymin,
			        densitymax = densitymax,
			        ylimmin = ylimmin,
			        ylimmax = ylimmax,
			        corrnan = corrnan,
			        synthetic = synthetic, 
			        iii = iii)

	print('  ')
	elapsed_time = time.time() - start_time
	m, s = divmod(elapsed_time, 60)
	h, m = divmod(m, 60)
	print(u'Finished in %d hours, %d minutes and %d seconds...' %  (h, m, s))
	print(u'_______________________________________________________________________________')
	print(u' ')
	print(u' ')


if __name__ == u'__main__':
	
	
	# initiate variables
	
	# run the transformation  
	swathp()

# END of program
# Pffffiouuuu


######!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

# Import of the lib pyswath
import pyswath

def readme():
	with open('README.rst') as f:
		return f.read()

setup(name='pyswath',
	#version='0.1.1',
	version=pyswath.__version__,
	description='package that provide tools to extract swath profiles from a raster map',
	long_descritpion=open('README.rst').read(),
	url='https://github.com/robertxa/pyswath',
	download_url='https://github.com/robertxa/pyswath/archive/master.zip',
	author='Xavier Robert',
	author_email='xavier.robert@univ-grenoble-alpes.fr',
	license='CCby-nc-sa',
	packages=find_packages(),
	install_requires=[
	      'rasterstats>=0.11.0',
	      'shapely',
	      'numpy',
	      'utm',
	      'gdal',
	      'progress'
	],
	classifiers=[
		"Operating System :: OS Independent",
		"Topic :: Scientific/Engineering :: GIS"
	],
	include_package_data=True,
	zip_safe=False)
      


######!/usr/bin/env python
# -*- coding: utf-8 -*-

# Do divisions with Reals, not with integers
# Must be at the beginning of the file
from __future__ import division
from __future__ import absolute_import
from __future__ import print_function

# To do :
#	- test other projections, I have not tested it
#   - mask the frequency outside of min/max
#   - remove holes from the dem... TO BE TESTED
#   - remove holes from the min file
# End To Do

###### CHANGES
#    -2015/08/07 : Correct the stepping if two types of profiles (with or without multipoints) are asked in the same run
#                  Add options to choose the size of the graphic
#                  Add options to force the density bar color scale
#    -2015/08/06 : Save a shapefile with the points of the profil
#    -2015/08/04 : Multipoint profils
#    -2015/07/30 : correct bug in function findline to take in account if B>A
#	 -2015/07/30 : remove NaN from graphs
#    -2015/09/15 : Add swath on synthetic DEM
#
######  End CHANGES


############################################################################

## Import Python modules
## I have problems to install rasterio : it does not find gdal libraries... from kingchaos
##modulesNames = ['sys', 'math', 'os', 'utm', 'warnings', 'rasterstats', 'shapely', 'copy', 'time', 'rasterio']
modulesNames = ['sys', 'os', 'warnings']
for module in modulesNames:
	try:
		# because we want to import using a variable, do it this way
		module_obj = __import__(module)
		# create a global object containging our module
		globals()[module] = module_obj
	except ImportError:
		#sys.exit(u"ERROR : Module " + module + " not present. \n\n Please, install it \
		raise ImportError(u"ERROR : Module " + module + " not present. \n\n Please, install it \
			      \n\n Edit the source code for more information")
from os import path, access, R_OK, mkdir         # W_OK for write permission.
try:
	import numpy as np                               # need version 1.7 or higher
except ImportError:
	#sys.exit(u"ERROR : Module Numpy not present. \n\n Please, install it \
	raise ImportError(u"ERROR : Module Numpy not present. \n\n Please, install it \
		      \n\n Edit the source code for more information")
try:
	from osgeo import gdal, gdalnumeric, ogr, osr    # For GIS operations
	from osgeo.gdalconst import *
except ImportError:
	#sys.exit(u"ERROR : Module osgeo/gdal not present. \n\n Please, install it \
	raise ImportError(u"ERROR : Module osgeo/gdal not present. \n\n Please, install it \
		      \n\n Edit the source code for more information")

from .raster_tools import *
from .profiles import *
from .plotgraph import *

############################################################################


def checkfiles(rasterfnme, A, B, xsteps, boxwidths, shpbox, title, 
				Coord, synthetic,
				multipoints, remNoData):	
	"""
	
	INPUTS
		- rasterfnme, 
		- A, B, 
		- xsteps, 
		- boxwidths, 
		- shpbox, 
		- title, 
		- Coord, 
		- synthetic,
		- multipoints, 
		- remNoData
	
	OUTPUTS
		- xsteps: 
		- boxwidths: 
		- Coord: 
		- srs_init: Initial coordinate system 
		- srs: Working coordinate systems
		- dst_filename: 
		- a, b: 
		- a_utm, b_utm: 
		- test_N:
		- shpbox: 
		- ulx, lrx, lry, uly: 
		- projdone:
		
	"""
	# Check if the DEM exists
	if not path.isfile(rasterfnme) or not access(rasterfnme, R_OK):
		# if not, exist
		raise NameError(u'ERROR: raster {FileNa} dem does not exist'.format(FileNa = rasterfnme))
		
	# Change list to numpy array
	a = np.array(A)
	b = np.array(B)
	a_utm = a
	b_utm = b
	#xstepsa = np.array(xsteps)
	#boxwidthsa = np.array(boxwidths)
	
	test_N = 1
	projdone = False
	
	# Check if the number of points A are the same of point B
	if len(xsteps) != 1 or len(boxwidths) != 1:
		if a.shape != b.shape \
		   or (a.shape[0] != len(xsteps) and len(xsteps) != 1) \
		   or (a.shape[0] != len(multipoints) and len(multipoints) != 1) \
		   or (a.shape[0] != len(boxwidths) and len(boxwidths) != 1) \
		   or (b.shape[0] != len(xsteps) and len(xsteps) != 1) \
		   or (b.shape[0] != len(boxwidths) and len(boxwidths) != 1) \
		   or (b.shape[0] != len(multipoints) and len(multipoints) != 1) \
		   or (len(xsteps) != len(boxwidths) and len(xsteps) != 1 and len(boxwidths) != 1 and len(multipoints) != 1):
			print(u'A =', A)
			print(u'B =', B)
			print(u'xsteps = ', xsteps)
			print(u'boxwidths = ', boxwidths)
			print(u'multipoints', multipoints)
			# If not, exist...
			raise NameError(u"ERROR : the number of points A and points B are different. \n" 
				     u"        Please, check your entries A, B, xsteps, boxwidth and multipoints! \n \n")
	
	# Change the name of the shapefile
	if shpbox[-4:] == '.shp':
		shpbox = 'SHP_' + title + '/' + shpbox[0:-4] + '_' + title + '.shp'
	else:
		shpbox = 'SHP_' + title + '/' + shpbox + '_' + title + '.shp'
	# Erase all the preexisting shapefiles of the project
	#if path.exists("SHP_" + title) == True and path.isfile(shpbox[0:-4] + '1.shp'):
	if path.exists(u"SHP_" + title) == True:
		# erase the preexisting shapefiles
		files = os.listdir("SHP_" + title)
		for f in files:
			full_path = os.path.join("SHP_" + title,f)
			if not os.path.isdir(full_path):
				os.remove(full_path)
	if path.exists("SHP_" + title) == False:
		os.mkdir("SHP_" + title) 
	if path.exists("TMP") == False:
		os.mkdir("TMP")
	if path.exists("TMP"):
		# erase the preexisting shapefile
		files = os.listdir("TMP")
		for f in files:
			full_path = os.path.join("TMP",f)
			if not os.path.isdir(full_path):
				os.remove(full_path) 
	if path.exists("Graphs") == False:
		os.mkdir("Graphs")
	if path.exists("Data") == False:
		os.mkdir("Data")
	
	dst_filename = rasterfnme
	
	print('  ')
	# Check if the xstep is greater than the distance between 2 pixels
	# Load the DEM
	print(u'Reading the DEM...')
	if remNoData:
		# Check if there are no data values
		srcband = rasterdata.GetRasterBand(1)
		# Get the caracteristics of the DEM
		dst_fnme = rasterfnme[0:-4] + '_fill.tif'
		if path.isfile(dst_fnme):
			print(u'file %s already exists...' % dst_fnme)
			rasterdata = gdal.Open(dst_fnme, GA_ReadOnly)
		else:
			rasterdataor = gdal.Open(rasterfnme, GA_ReadOnly)
			rasterdata = removeNoData(rasterdataor, dst_fnme)
	else:
		rasterdata = gdal.Open(rasterfnme, GA_ReadOnly)
	
	geotransform = rasterdata.GetGeoTransform()
	# Check if xteps is compatible with the DEM resolution
	for iii in range (0, len(xsteps)):
		if min(geotransform[1], geotransform[5]) >= xsteps[iii]:
			warnings.warn(u'Xstep smaller that the pixel size of the DEM ! \n \
	    		            Xstep is set to 3 times the pixel size')
			xsteps[iii] = max(geotransform[1], geotransform[5]) * 3
	# xtep is now OK for processing
	
	# Check if A and B are in the DEM region or not
	# Calcul the extent of the DEM
	(ulx, uly) = (geotransform[0], geotransform[3])
	(lrx, lry) = (geotransform[0] + geotransform[1] * rasterdata.RasterXSize,
		          geotransform[3] + geotransform[5] * rasterdata.RasterYSize)	
	for i in range (0, a.shape[0]):
		if  not (ulx < A[i][0]< lrx):
			raise NameError(u"ERROR : A%s is outside the DEM border %s" % (A[i], (ulx, lrx)))
		if  not (lry < A[i][1]< uly):
			raise NameError(u"ERROR : A%s is outside the DEM border %s" % (A[i], (lry, uly)))
		if  not (ulx < B[i][0]< lrx):
			raise NameError(u"ERROR : B%s is outside the DEM border %s" % (A[i], (ulx, lrx)))
		if  not (lry < B[i][1]< uly):
			raise NameError(u"ERROR : B%s is outside the DEM border %s" % (A[i], (lry, uly)))
	# Check if the raster is in UTM
	# get projection of the raster source	
	src_proj = rasterdata.GetProjection()
	srs = osr.SpatialReference(wkt = src_proj)
	
	# Keep the original srs for future calculs
	srs_init = srs
	
	# To find the projection : srs.GetAttrValue('geogcs')
	if not synthetic and ('UTM' not in src_proj or (Coord[0:3] != 'utm' and Coord[0:3] != 'UTM')):
		projdone = True
		src_geotrans = geotransform
		print(u'The Dem is not projected.')
		print(u'Doing the projection to UTM...')
		print(u'It could be long...')            
		#xstep, boxwidth, Coord, srs, dst_filename, a_utm, b_utm, test_N = \
		xsteps, boxwidths, Coord, srs, dst_filename, a_utm, b_utm, test_N = \
		       project_raster(rasterfnme, 
		                      rasterdata, 
		                      src_geotrans, 
		                      srs, 
		                      A, 
		                      B, 
		                      Coord, 
		                      xsteps, 
		                      boxwidths)
	elif synthetic:
		print(u'Working on a synthetic DEM...')
		text_N = 1
	else:
		if 'south' in srs.ExportToProj4():
			text_N = 0
		else:
			text_N = 1	
	
	return xsteps, boxwidths, Coord, srs_init, srs, dst_filename, a, b, a_utm, b_utm, test_N, shpbox, ulx, lrx, lry, uly, projdone
	
	


######!/usr/bin/env python
# -*- coding: utf-8 -*-

# Do divisions with Reals, not with integers
# Must be at the beginning of the file
from __future__ import division
from __future__ import absolute_import
from __future__ import print_function

## Import Python modules
## I have problems to install rasterio : it does not find gdal libraries... from kingchaos
modulesNames = ['sys']
for module in modulesNames:
	try:
		# because we want to import using a variable, do it this way
		module_obj = __import__(module)
		# create a global object containging our module
		globals()[module] = module_obj
	except ImportError:
		raise ImportError(u"ERROR : Module %s not present. \n\n Please, install it \
			      \n\n Edit the source code for more information" %(module))
try:
	import numpy as np                               # need version 1.7 or higher
except ImportError:
	raise ImportError(u"ERROR : Module Numpy not present. \n\n Please, install it \
		      \n\n Edit the source code for more information")
try:
	import matplotlib.pyplot as plt                  # module to plot figures
	from matplotlib import cm, rc
except ImportError:
	raise ImportError(u"ERROR : Module matplotlib not present. \n\n Please, install it \
		      \n\n Edit the source code for more information")

############################################################################
	
def plot_graph(data, datamask, statslines, title, xdist, xstep, boxwidth, factor, iii, corrnan, \
               sizeplotx = None, sizeploty = None, densitymin = None, densitymax = None,
               ylimmin = None, ylimmax = None, synthetic = False, xpoint = None):
	"""
	Function to plot swath profile
	
	INPUTS:
	   data = frequency of the altitude
	   datamask = mask the data where it is null (set null values to nan)
	              used to build the frequency range
	   statslines = min, mac, mean, median data to plot
	   title = title of the graph
	   xdist = length of the transect
	   factor = factor to apply to plot the x-axes in km
	   corrnan = Flag to correct (True) or no (False) the graph from the Nan values
	   synthetic = True if synthetic dem
	   xpoint = distance along profile of the intermediary points if it exists
	
	OUTPUTS:
	   Graph in pdf
	
	USAGE:
	   plot_graph(data, datamask, statslines, title, xdist, factor)
	
	"""	
	# Do the plot:
	print(u'   Plot the transect...')	
	
	linewidth = 1
	plt.clf()
	
	# Change the settings of the graph
	if sizeplotx and sizeploty:
		plt.figure(num = None, figsize=(sizeplotx,sizeploty))
	
	## plot the altitude density plot in red
	plt.hexbin(data[:,0] / factor, data[:,1], C = datamask, cmap = cm.Reds, bins = None)
	## make a color palette with the altitude frequency
	# to force the boundaries of the color palette :
	if densitymin and densitymax:
		plt.clim(densitymin, densitymax)
	cd = plt.colorbar()
	cd.set_label(u'Altitude frequency')
	
	#if synthetic:
	#	factor = 1 / factor
	
	# plot the max, the mean, and the min altitude along the profile
	#     You can change from mean to median altitude by commenting the plt.plot mean block
	#     and de-commenting the plt.plot Median plot
	plt.plot(statslines[:,0] / factor, statslines[:,2], 
	         color = 'blue', 
	         linestyle = '-',
	         linewidth = linewidth, 
	         label = u'Max alt')
	#plt.plot(statslines[:,0], statslines[:,3], 
	#         color = 'black', 
	#         linestyle = '-', 
	#         linewidth = 2
	#         label = 'Median alt')
	
	plt.plot(statslines[:,0] / factor, statslines[:,4], 
	         color = 'black', 
	         linewidth = 2, 
	         linestyle = '-', 
	         label = u'Mean alt')
	
	# remove NoData (valid only for the min ; I assume that NaN data are data inf. or equal to 0)
	# TO BE CHANGED	!!!!!!
	statslines[statslines[:,1] <= 0.0, 1] = np.nan
	
	#altifreq = np.ma.masked_where(data[:,2] == 0, data[:,1])
	altifreq1 = data
	altifreq1[altifreq1[:,2] == 0.0, 1] = np.nan
	altifreq1[altifreq1[:,1] == 0.0, 1] = np.nan
	altifreq = altifreq1[:,1]
	
	if corrnan:
		ploty = np.zeros(statslines.shape[0])
		for k in range (0, ploty.shape[0]):
			for jj in range (int((k-1) * altifreq.shape[0]/statslines.shape[0]), 
			                 int(k * altifreq.shape[0]/statslines.shape[0])):
				if np.isnan(altifreq[jj]):
					altifreq[jj] = altifreq[jj+1]
			
			altifreqmin = np.nanmin(altifreq[k * altifreq.shape[0]/statslines.shape[0] : 
			                           (k +1) * altifreq.shape[0]/statslines.shape[0]])
			if not np.isnan(statslines[k,1]):
				ploty[k] = min(statslines[k,1], altifreqmin)
			else:
				ploty[k] = altifreqmin
		
		plt.plot(statslines[:,0] / factor, ploty, 
		#plt.plot(statslines[:,0] / factor, statslines[:,1], 
	    	     color = 'green', 
	        	 linestyle = '-', 
	        	 linewidth = linewidth,
		         label = u'Min alt')
	else:
		 plt.plot(statslines[:,0] / factor, statslines[:,1], 
	    	     color = 'green', 
	        	 linestyle = '-', 
	        	 linewidth = linewidth,
		         label = u'Min alt')
	if xpoint:
		for jjj in range(0,len(xpoint)):
			plt.axvline(x = xpoint[jjj]/factor, ls = '--', linewidth = 1, color = '0.75')#, label = 'STDS position')
	
	#plt.ylim(0,statslines[:,2].max() + 500)
	if ylimmax:
		if ylimmin:
			plt.ylim(ylimmin, ylimmax)
		else:
			plt.ylim(0, ylimmax)
	else:
		if ylimmin:
			plt.ylim(ylimmin, statslines[:,2].max())
		else:
			plt.ylim(0, statslines[:,2].max())
	
	plt.xlim(0,xdist)
	
	plt.xlabel(u'Distance along profile (km)')
	plt.ylabel(u'Altitude (m)')
	plt.legend(loc='best', numpoints = 1)
	
	plt.title(r'%s %s (Xstep = %s km; Boxwidth = %s km)'
	          %(title, str(iii + 1), str(round((xstep / factor), 2)), str(round((boxwidth / factor), 2))))
	plt.savefig("Graphs/" + title + '_transect_' + str(iii + 1) + '.pdf')
	plt.close()
	print(u'   Saved in Graphs/%s_transect_%s.pdf'
	      %(title, str(iii + 1)))

	return
	